use crate::save_identification::{Identification, IdentificationGateway};
use std::net::SocketAddr;

#[derive(Debug)]
pub struct SaveIdentificationCommand {
    forwarded_for: Option<String>,
    user_agent: Option<String>,
    proxy_user: Option<String>,
    ip_address: Option<String>,
}

impl SaveIdentificationCommand {
    pub fn new(
        forwarded_for: Option<&str>,
        user_agent: Option<&str>,
        proxy_user: Option<&str>,
        ip_address: Option<SocketAddr>,
    ) -> SaveIdentificationCommand {
        SaveIdentificationCommand {
            forwarded_for: match forwarded_for {
                Some(x) => Some(x.to_string()),
                _ => None,
            },
            user_agent: match user_agent {
                Some(x) => Some(x.to_string()),
                _ => None,
            },
            proxy_user: match proxy_user {
                Some(x) => Some(x.to_string()),
                _ => None,
            },
            ip_address: match ip_address {
                Some(x) => Some(x.to_string()),
                _ => None,
            },
        }
    }
}

impl From<SaveIdentificationCommand> for Identification {
    fn from(command: SaveIdentificationCommand) -> Self {
        Identification {
            forwarded_for: command.forwarded_for,
            user_agent: command.user_agent,
            proxy_user: command.proxy_user,
            ip_address: command.ip_address,
        }
    }
}

pub struct SaveIdentificationCommandHandler {
    save_identification_gateway: dyn IdentificationGateway,
}

impl SaveIdentificationCommandHandler {
    pub fn save_identification(&self, command: SaveIdentificationCommand) {
        let identification: Identification = command.into();
    }
}
