use std::net::SocketAddr;
use std::path::{Path, PathBuf};
use rocket::fs::{NamedFile, relative};
use rocket::http::{RawStr, Status};
use rocket::Request;
use rocket::request::{FromRequest, Outcome};
use rocket::response::{content, status};
use rocket::response::status::NotFound;
use rocket::routes;
use rocket::get;
use rocket::Rocket;
use rocket::Build;
use crate::guards::IdentificationGuard;

pub fn launch_server() -> Rocket<Build> {
    rocket::build().mount("/", routes![files])
}

#[get("/image/<file..>")]
async fn files(identification_guard: IdentificationGuard, file: PathBuf) -> Option<NamedFile> {
    println!("identification: {:?}", identification_guard.identification);
    NamedFile::open(Path::new("static/").join(file)).await.ok()
}

mod guards {
    use async_trait::async_trait;
    use rocket::Request;
    use rocket::request::{FromRequest, Outcome};
    use kernel::save_identification::Identification;

    #[derive(Debug)]
    pub struct IdentificationGuard {
        pub identification: Identification,
    }

    #[async_trait]
    impl<'a> FromRequest<'a> for IdentificationGuard {
        type Error = ();

        async fn from_request(request: &'a Request<'_>) -> Outcome<IdentificationGuard, Self::Error> {
            let forwarded_for = request.headers().get_one("x-forwarded-for");
            let user_agent = request.headers().get_one("user-agent");
            let proxy_user = request.headers().get_one("x-proxyuser");
            let ip_address = request.remote();

            let identity_guard = IdentificationGuard { identification: Identification::new(forwarded_for, user_agent, proxy_user, ip_address) };

            println!("{:?}", identity_guard);
            Outcome::Success(identity_guard)
        }
    }
}
